@@ -29,13 +29,16 @@ static long timebase;		      /* Time in seconds at start of program */
 static struct sockaddr_in lookhost;   /* Look who's listening host, if any */
 static char *sdes = NULL;	      /* RTP SDES packet */
 static int sdesl;		      /* RTP SDES packet length */
-static int lwlport = Internet_Port + 2; /* Look Who's Listening port */
 static unsigned long ssrc;	      /* RTP synchronisation source identifier */
 static double lwltimer; 	      /* Time next LWL retransmit scheduled */
 static int actives = 0; 	      /* Currently active hosts */
 
 #define LWL_RETRANSMIT	(10 * 60 * 1000000.0) /* Microseconds between LWL updates */
 
+static struct in_addr lwl_sites[LWL_MAX_SITES]; /* LWL site addresses */
+static long lwl_ports[LWL_MAX_SITES]; /* Ports for LWL hosts */
+static int lwl_nsites = 0;	      /* Number of LWL sites published on */
+
 /* Open connection state. */
 
 #define connection e_connection
@@ -199,10 +202,19 @@ static char *compressionType(msg, clue)
 	   ((msg->compression & (fComp2X | fCompLPC)) ==
 				(fComp2X | fCompLPC)) ?
                                 "LPC+2X compressed" :
+	   ((msg->compression & (fComp2X | fCompLPC10)) ==
+				(fComp2X | fCompLPC10)) ?
+                                "LPC10+2X compressed" :
+	   ((msg->compression & (fComp2X | fCompVOX)) ==
+				(fComp2X | fCompVOX)) ?
+                                "VOX+2X compressed" :
            ((msg->compression & fCompADPCM) ? "ADPCM compressed" :
            ((msg->compression & fCompLPC) ? "LPC compressed" :
+           ((msg->compression & fCompLPC10) ? "LPC10 compressed" :
            ((msg->compression & fComp2X) ? "2X compressed" :
-           ((msg->compression & fCompGSM) ? "GSM compressed" :  "uncompressed"))));
+           ((msg->compression & fCompGSM) ? "GSM compressed" :
+           ((msg->compression & fCompVOX) ? "VOX compressed" :
+                                            "uncompressed"))))));
 }
 
 /*  MAKESESSIONKEY  --	Generate session key.  */
@@ -249,37 +261,47 @@ static void ularm(t)
 }
 
 /*  SENDLWLMESSAGE  --	If enabled, send a message identifying us
-                        to the selected Look Who's Listening server.  */
+                        to each selected Look Who's Listening server.  */
 
 static void sendLwlMessage(dobye)
   int dobye;
 {
-    int sock;
+    int i, sock;
+    struct sockaddr_in name;
 
-    sock = socket(AF_INET, SOCK_STREAM, 0);
-    if (sock < 0) {
-        perror("opening look who's listening socket");
-	sdes = NULL;
-	return;
-    }
-
-    if (connect(sock, (struct sockaddr *) &(lookhost), sizeof lookhost) >= 0) {
-	if (dobye) {
-	    char v[1024];
-	    int l;
-
-            l = rtp_make_bye(v, ssrc, "Exiting sfspeaker", FALSE);
-	    if (send(sock, v, l, 0) < 0) {
-                perror("sending look who's listening BYE packet");
+    for (i = 0; i < lwl_nsites; i++) {
+	if (lwl_ports[i] >= 0) {
+	    sock = socket(AF_INET, SOCK_STREAM, 0);
+	    if (sock < 0) {
+                perror("opening look who's listening socket");
+		sdes = NULL;
+		return;
 	    }
-	} else {
-	    if (send(sock, (char *) sdes, sdesl, 0) < 0) {
-                perror("sending look who's listening source ID message");
+
+	    lookhost.sin_port = htons(lwl_ports[i]);
+	    bcopy((char *) (&lwl_sites[i]), (char *) &lookhost.sin_addr.s_addr,
+		  sizeof lookhost.sin_addr.s_addr);
+
+	    if (connect(sock, (struct sockaddr *) &(lookhost), sizeof lookhost) >= 0) {
+		if (dobye) {
+		    char v[1024];
+		    int l;
+
+                    l = rtp_make_bye(v, ssrc, "Exiting sfspeaker", FALSE);
+		    if (send(sock, v, l, 0) < 0) {
+                        perror("sending look who's listening BYE packet");
+		    }
+		} else {
+		    if (send(sock, (char *) sdes, sdesl, 0) < 0) {
+                        perror("sending look who's listening source ID message");
+		    }
+		}
+	    } else {
+                perror("connecting look who's listening socket");
 	    }
 	}
-    } else {
-        perror("connecting look who's listening socket");
     }
+    lwltimer = LWL_RETRANSMIT;
     close(sock);
 }
 
@@ -398,6 +420,16 @@ static void exiting()
     exit(0);
 }
 
+/*  PROG_NAME  --  Extract program name from argv[0].  */
+
+static char *prog_name(arg)
+  char *arg;
+{
+    char *cp = strrchr(arg, '/');
+
+    return (cp != NULL) ? cp + 1 : arg;
+}
+
 /*  USAGE  --  Print how-to-call information.  */
 
 static void usage()
@@ -436,7 +468,7 @@ main(argc, argv)
     struct timezone tzp;
     struct queuedPacket *qp;
 
-    prog = argv[0];
+    prog = prog_name(argv[0]);
     gettimeofday(&tp, &tzp);
     timebase = tp.tv_sec;
 
@@ -554,28 +586,58 @@ main(argc, argv)
     if (cp != NULL) {
 	struct hostent *h;
 	char md5key[16];
-	char *ep;
+	char *ep, *np;
+	long iadr;
+	int n;
 
-        if ((ep = strchr(cp, ':')) != NULL) {
-	    *ep = 0;
-	    lwlport = atoi(ep + 1);
+	makeSessionKey(md5key);
+	bcopy(md5key, (char *) &ssrc, sizeof ssrc);
+	sdesl = rtp_make_sdes(&sdes, ssrc, whichport, FALSE);
+	lookhost.sin_family = AF_INET;
+
+	while (lwl_nsites < LWL_MAX_SITES) {
+	    n = lwl_nsites;
+	    while (*cp != 0 && isspace(*cp)) {
+		cp++;
+	    }
+	    if (*cp == 0) {
+		break;
+	    }
+            if ((np = strchr(cp, ',')) != NULL) {
+		*np++ = 0;
+	    }
+	    lwl_ports[lwl_nsites] = Internet_Port + 2;
+            if ((ep = strchr(cp, ':')) != NULL) {
+		*ep = 0;
+		lwl_ports[lwl_nsites] = atoi(ep + 1);
+	    }
+	    if (isdigit(*cp) && (iadr = inet_addr(cp)) != -1) {
+		bcopy((char *) &iadr, (char *) (&lwl_sites[lwl_nsites]),
+		      sizeof iadr);
+		lwl_nsites++;
+	    } else {
+		h = gethostbyname(cp);
+		if (h != NULL) {
+		    bcopy((char *) (h->h_addr), 
+			  (char *) (&lwl_sites[lwl_nsites]),
+			  sizeof(unsigned long));
+		    lwl_nsites++;
+		} else {
+                    fprintf(stderr, "%s: warning, SPEAKFREE_LWL_TELL host %s unknown.\n",
+			prog, cp);
+		}
+	    }
+	    if (Debug && lwl_nsites > n) {
+                fprintf(stderr, "%s: publishing on LWL server %s: %s.\n", prog,
+		    inet_ntoa(lwl_sites[n]), cp);
+	    }
+	    if (np == NULL) {
+		break;
+	    }
+	    cp = np;
 	}
-	h = gethostbyname(cp);
-	if (h != NULL) {
-	    bcopy((char *) (h->h_addr), (char *) &lookhost.sin_addr.s_addr,
-		sizeof lookhost.sin_addr.s_addr);
-	    lookhost.sin_family = AF_INET;
-	    lookhost.sin_port = htons(lwlport);
-
-	    makeSessionKey(md5key);
-	    bcopy(md5key, (char *) &ssrc, sizeof ssrc);
-
-	    sdesl = rtp_make_sdes(&sdes, ssrc, whichport, FALSE);
+	if (lwl_nsites > 0) {
 	    sendLwlMessage(FALSE);
-	    lwltimer = LWL_RETRANSMIT + tp.tv_usec;
-	} else {
-            fprintf(stderr, "%s: warning, SPEAKFREE_LWL_TELL host %s unknown.\n",
-		prog, cp);
 	}
     }
 
