@@ -10,7 +10,7 @@
 #define TockTock    120 	      /* Timeout check frequency, seconds */
 #define TimeoutTime 15 * 60	      /* No-response timeout interval, seconds */
 
-#define MaxReplyPacket 400	      /* Maximum length of reply data */
+#define MaxReplyPacket 512	      /* Maximum length of reply data */
 
 static int sock;		      /* Our socket */
 static char *prog;		      /* Program name */ 
@@ -30,6 +30,11 @@ static int htmlChange = TRUE;	      /* Change since last HTML update ? */
 static char *message = NULL;	      /* Server information message */
 static int messagel;		      /* Length of server information message */
 
+static struct sockaddr_in lookhost;   /* Look who's listening host, if any */
+static struct in_addr lwl_sites[LWL_MAX_SITES]; /* LWL site addresses */
+static long lwl_ports[LWL_MAX_SITES]; /* Ports for LWL hosts */
+static int lwl_nsites = 0;	      /* Number of LWL sites published on */
+
 struct lwl {
     struct lwl *next;		      /* Next connection */
     time_t ltime;		      /* Time of last update */
@@ -188,6 +193,32 @@ static void logLwl(lw, event)
     printf("%s %s%s\n", estime(), event, deef);
 }
 
+/*  GARDOL  -- Validity-check an RTCP packet to make sure we don't
+	       crash if some bozo sends us garbage.  */
+
+static int gardol(p, len)
+  unsigned char *p;
+  int len;
+{
+    unsigned char *end;
+
+    if ((((p[0] >> 6) & 3) != RTP_VERSION) ||	   /* Version incorrect ? */
+	((p[0] & 0x20) != 0) || 		   /* Padding in first packet ? */
+	((p[1] != RTCP_SR) && (p[1] != RTCP_RR) &&
+	 (p[1] != RTCP_SDES) && (p[1] != RTCP_BYE) &&
+	 (p[1] != RTCP_APP))) { 		   /* Item type valid ? */
+	return FALSE;
+    }
+    end = p + len;
+
+    do {
+	/* Advance to next subpacket */
+	p += (ntohs(*((short *) (p + 2))) + 1) * 4;
+    } while (p < end && (((p[0] >> 6) & 3) == RTP_VERSION));
+
+    return p == end;
+}
+
 /*  MAKEHTML  --  Create an HTML file showing current connections.  If
                   "private" is set, exact-match names are included
 		  in the HTML file; this allows creation of a non-exported
@@ -319,6 +350,117 @@ static void lcase(s)
     }
 }
 
+/*  FORWARDLIST  --  Parse a list of servers to forward packets to. */
+
+static void forwardList(cp)
+  char *cp;
+{
+    struct hostent *h;
+    char md5key[16];
+    char *ep, *np;
+    long iadr;
+    int n;
+
+    lookhost.sin_family = AF_INET;
+
+    while (lwl_nsites < LWL_MAX_SITES) {
+	n = lwl_nsites;
+	while (*cp != 0 && isspace(*cp)) {
+	    cp++;
+	}
+	if (*cp == 0) {
+	    break;
+	}
+        if ((np = strchr(cp, ',')) != NULL) {
+	    *np++ = 0;
+	}
+	lwl_ports[lwl_nsites] = Internet_Port + 2;
+        if ((ep = strchr(cp, ':')) != NULL) {
+	    *ep = 0;
+	    lwl_ports[lwl_nsites] = atoi(ep + 1);
+	}
+	if (isdigit(*cp) && (iadr = inet_addr(cp)) != -1) {
+	    bcopy((char *) &iadr, (char *) (&lwl_sites[lwl_nsites]),
+		  sizeof iadr);
+	    lwl_nsites++;
+	} else {
+	    h = gethostbyname(cp);
+	    if (h != NULL) {
+		bcopy((char *) (h->h_addr), 
+		      (char *) (&lwl_sites[lwl_nsites]),
+		      sizeof(unsigned long));
+		lwl_nsites++;
+	    } else {
+                fprintf(stderr, "%s: warning, forward destination %s unknown.\n",
+		    prog, cp);
+	    }
+	}
+	if (debugging && lwl_nsites > n) {
+            fprintf(stderr, "%s: forwarding to LWL server %s: %s.\n", prog,
+		inet_ntoa(lwl_sites[n]), cp);
+	}
+	if (np == NULL) {
+	    break;
+	}
+	cp = np;
+    }
+}
+
+/*  FORWARDLWLMESSAGE  --  If forwarding is enabled, pass this message
+			   on to hosts in our forward list, changing the
+			   packet protocol header to indicate the packet
+			   was forwarded and appending the IP address
+			   of the originating host to the end of the packet. */
+
+static void forwardLwlMessage(zp, rll, addr)
+  char *zp;			/* Packet address */
+  int rll;			/* Total packet length (starting at zp) */
+  struct in_addr addr;		/* IP address of originator of this packet */
+{
+    int i, sock, wasfwd = FALSE;
+
+    zp[0] |= 0x40;		/* Set forwarded packet flag bit */
+    bcopy(&addr, zp + rll, sizeof(struct in_addr));
+    rll += sizeof(struct in_addr);
+    for (i = 0; i < lwl_nsites; i++) {
+	if (lwl_ports[i] >= 0) {
+	    sock = socket(AF_INET, SOCK_STREAM, 0);
+	    if (sock < 0) {
+                perror("opening forwarding socket");
+		return;
+	    }
+
+	    lookhost.sin_port = htons(lwl_ports[i]);
+	    bcopy((char *) (&lwl_sites[i]), (char *) &lookhost.sin_addr.s_addr,
+		  sizeof lookhost.sin_addr.s_addr);
+
+	    if (connect(sock, (struct sockaddr *) &(lookhost), sizeof lookhost) >= 0) {
+		if (send(sock, zp, rll, 0) < 0) {
+                    perror("forwarding look who's listening source ID message");
+		} else {
+		    if (debugging) {
+                        fprintf(stderr, "%s: forwarded packet to server %s.\n",
+			    prog, inet_ntoa(lookhost.sin_addr));
+		    }
+		}
+	    } else {
+
+		/* As LWL servers go up and down, failures to connect
+		   to forwarding destinations are not uncommon.  Further,
+		   while a server is down, we get here on every packet
+		   we attempt to forward.  To avoid blithering all
+		   over the log, we only note such failure if in
+		   prolix or debugging mode. */
+
+		if (prolix || debugging) {
+                    perror("connecting forwarding socket");
+		}
+	    }
+	}
+    }
+    close(sock);
+}
+
 /*  QUERYMATCH	--  Determing if query matches a given item.  */
 
 static int queryMatch(q, l)
@@ -434,6 +576,16 @@ fprintf(stderr, "Exiting.\n");
     exit(0);
 }
 
+/*  PROG_NAME  --  Extract program name from argv[0].  */
+
+static char *prog_name(arg)
+  char *arg;
+{
+    char *cp = strrchr(arg, '/');
+
+    return (cp != NULL) ? cp + 1 : arg;
+}
+
 /*  USAGE  --  Print how-to-call information.  */
 
 static void usage()
@@ -444,6 +596,7 @@ static void usage()
     V fprintf(stderr, "Usage: %s [options]\n", prog);
     V fprintf(stderr, "Options:\n");
     V fprintf(stderr, "           -D         Debugging output on stderr\n");
+    V fprintf(stderr, "           -Fserv,... Forward to listed servers\n");
     V fprintf(stderr, "           -Hpath     Write HTML status on base path\n");
     V fprintf(stderr, "           -Insec     Interval between HTML updates\n");
     V fprintf(stderr, "           -Mfile     Load server message from file\n");
@@ -460,6 +613,117 @@ static void usage()
     V fprintf(stderr, "   WWW:    http://www.fourmilab.ch/\n");
 }
 
+/*  ADD_SDES_ITEM  --  Include an SDES item in a reply packet,
+		       pruning illegal characters and content,
+		       checking for packet overflow, and updating
+		       the packet pointer.  */
+
+#define addSDES(item, text) add_sdes_item(item, text, &ap)
+
+static int add_sdes_item(item, text, app)
+  int item;
+  char *text;
+  char **app;
+{
+    int l, state;
+    char prune[258];
+    char *ap = *app, *hp, *op = prune;
+
+    /* Copy the text to the "prune" buffer using a simple
+       state machine parser, filtering extraneous
+       information. */
+
+    state = 0;
+
+    while (*text) {
+	unsigned char ch = (unsigned char) *text++;
+
+	if (item == RTCP_SDES_PRIV && op == prune) {
+	    /* Allow PRIV item binary prefix field to pass. */
+	    *op++ = (char) ch;
+	    continue;
+	}
+
+	/* Discard non-graphic characters (ISO 8859 standard). */
+
+        if (!isspace(ch) && (ch < ' ' || (ch > '~' && ch < 161))) {
+	    continue;
+	}
+
+	switch (state) {
+
+	    case 0:		      /* Skipping initial white space */
+		if (!isspace(ch)) {
+		    state = 1;
+		} else {
+		    break;
+		}
+		/* Note fall-through */
+
+	    case 1:		      /* Transcribing to output */
+s1:             if (ch == '<') {
+                    if ((hp = strchr(text, '>')) != NULL) {
+			text = hp + 1;
+			if (op > prune && isspace(op[-1])) {
+			    state = 2;
+			}
+			continue;
+		    }
+		}
+		if (isspace(ch)) {
+                    ch = ' ';         /* Convert all white space into blanks */
+		    state = 2;
+		}
+		*op++ = (char) ch;
+		break;
+
+	    case 2:		      /* Collapsing multiple white space */
+		if (!isspace(ch)) {
+		    state = 1;
+		    goto s1;
+		}
+		break;
+	}
+    }
+    *op = 0;
+
+    /* Drop trailing blanks. */
+
+    while (strlen(prune) > 0 && isspace(op[-1])) {
+	*(--op) = 0;
+    }
+
+    l = strlen(prune);
+    if (l == 0) {
+	return TRUE;
+    } else if (l > 48) {
+	int n;
+
+	/* String is too long.	Try to trim it at a word boundary,
+	   but if that fails just brutally lop it off. */
+
+	for (n = 47; n > 31; n--) {
+	    if (isspace(prune[n])) {
+		prune[n] = 0;
+		n = 0;
+		break;
+	    }
+	}
+	if (n > 0) {
+	    prune[47] = 0;
+	}
+    }
+
+    *ap++ = item;
+
+    *ap++ = l = strlen(prune);
+    bcopy(prune, ap, l);
+    ap += l;
+
+    *app = ap;
+}
+
+
 /*  Main program.  */
 
 main(argc, argv)
@@ -471,7 +735,27 @@ main(argc, argv)
 
     /*	Process command line options.  */
 
-    prog = argv[0];
+    prog = prog_name(argv[0]);
+
+#ifdef PRUNEFACE
+    {	/* Interctive debugging of text pruning logic. */
+	char s[132], p[132];
+	char *zap;
+
+	while (TRUE) {
+            printf("--> ");
+	    if (fgets(s, sizeof s, stdin) == NULL) {
+		break;
+	    }
+	    zap = p;
+	    add_sdes_item(1, s, &zap);
+	    *zap = 0;
+            printf(    "\"%s\"\n", p + 2);
+	}
+	exit(0);
+    }
+#endif
+
     for (i = 1; i < argc; i++) {
 	char *op, opt;
 
@@ -488,6 +772,10 @@ main(argc, argv)
 		    debugging = TRUE;
 		    break;
 
+                case 'F':             /* -Fserv,...  --  Forward to listed servers. */
+		    forwardList(op + 1);
+		    break;
+
                 case 'H':             /* -Hname  --  HTML file name base path */
 		    htmlFile = op + 1;
 		    break;
@@ -632,14 +920,14 @@ main(argc, argv)
     /* Process requests from the socket. */
 
     while (TRUE) {
-	int csock, rll;
+	int csock, rll, forward, pvalid;
 	char zp[1024];
 	rtcp_t *rp;
 	short srp;
-	struct sockaddr_in from;      /* Sending host address */
+	struct sockaddr_in from, fwdh; /* Sending host address */
 	int fromlen;		      /* Length of sending host address */
 	struct lwl *lw; 	      /* Parsed packet */
-	char *p;
+	char *p, *pend;
 
 	errno = 0;
 	do {
@@ -657,6 +945,7 @@ main(argc, argv)
             printf("%s: %s accept\n", prog, inet_ntoa(from.sin_addr));
 	}
 
+	forward = FALSE;
 	rll = recv(csock, zp, sizeof zp, 0);
 #ifdef HEXDUMP
 	if (hexdump) {
@@ -665,23 +954,69 @@ main(argc, argv)
 	}
 #endif
 
+	/* If packet was forwarded from another server, extract the
+	   embedded original source address and make th the packet
+	   look like it came directly from the sender. */
+
+	if (rll > 4 && (zp[0] & 0xC0) == 0xC0) {
+	    forward = TRUE;
+	    zp[0] &= ~0x40;
+	    bcopy(zp + (rll - 4), &fwdh.sin_addr, 4);
+	    rll -= 4;
+	    if (debugging) {
+                fprintf(stderr, "%s: received forwarded packet from %s.\n",
+		    prog, inet_ntoa(fwdh.sin_addr));
+	    }
+	    from.sin_addr = fwdh.sin_addr ;
+	}
+
+        /* Some version of Speak Freely for Windows don't always
+	   pad request packets to a multiple of four bytes.  If
+	   this is the case, insert the pad ourselves. */
+
+	while (((zp[1] & 0xFF) == RTCP_APP) && ((rll & 3) != 0)) {
+	    zp[rll++] = 0;
+	}
+
+	/* Validate this packet.  If it fails the validity check,
+           ignore it do it can't crash the code below. */
+
+	if (!gardol(zp, rll)) {
+	    if (debugging) {
+                fprintf(stderr, "%s: discarded invalid RTCP packet from %s.\n",
+		    prog, inet_ntoa(from.sin_addr));
+	    }
+	    continue;
+	}
+
 	/* Walk through the individual items in a possibly composite
            packet until we locate an item we're interested in.  This
 	   allows us to accept packets that comply with the RTP standard
 	   that all RTCP packets begin with an SR or RR.  */
 
 	p = zp;
+	pend = p + rll;
+	pvalid = FALSE;
 
-	while ((p[0] >> 6 & 3) == RTP_VERSION) {
+	while ((p < pend) && (p[0] >> 6 & 3) == RTP_VERSION) {
 	    int pt = p[1] & 0xFF;
 
 	    if (pt == RTCP_SDES || pt == RTCP_BYE || pt == RTCP_APP) {
+		pvalid = TRUE;
 		break;
 	    }
 	    /* If not of interest to us, skip to next subpacket. */
 	    p += (ntohs(*((short *) (p + 2))) + 1) * 4;
 	}
 
+	if (!pvalid) {
+	    if (debugging) {
+                fprintf(stderr, "%s: discarded RTCP packet with unknown payload from %s.\n",
+		    prog, inet_ntoa(from.sin_addr));
+	    }
+	    continue;
+	}
+
 	/* Examine the packet to see what kind of request it is.
            Note that since all the packets we're interested in can
 	   be parsed without knowing their actual length in bytes, we
@@ -841,6 +1176,18 @@ main(argc, argv)
 				    }
 				    changed();
 				}
+				if (!forward && lwl_nsites > 0) {
+				    forwardLwlMessage(zp, rll, from.sin_addr);
+				}
+			    } else {
+				/* Bogus item with no CNAME -- discard. */
+				if (debugging || verbose) {
+                                    fprintf(stderr, "Bogus SDES with no CNAME.\n");
+				    if (!debugging) {
+					dumpLwl(stderr, lw);
+				    }
+				}
+				destroyLwl(lw);
 			    }
 			}
 		    }
@@ -883,6 +1230,9 @@ main(argc, argv)
 			    llr = lr;
 			    lr = lr->next;
 			}
+			if (!forward && lwl_nsites > 0) {
+			    forwardLwlMessage(zp, rll, from.sin_addr);
+			}
 		    }
 		    break;
 
@@ -908,7 +1258,7 @@ main(argc, argv)
 			struct lwl *lr = conn;
 			char b[1500];
 			rtcp_t *rp = (rtcp_t *) b;
-			char *ap;
+			char *ap, *pap;
 			int l, scandex = 0;
 
 #ifdef RationalWorld
@@ -931,9 +1281,7 @@ main(argc, argv)
 			    if (queryMatch(p + 12, lr)) {
 				char s[20];
 
-#define addSDES(item, text) *ap++ = item; *ap++ = l = strlen(text); \
-			    bcopy(text, ap, l); ap += l
-
+				pap = ap;
 				*((unsigned long *) ap) = lr->ssrc;
 				ap += sizeof(unsigned long);
 				addSDES(RTCP_SDES_CNAME, lr->cname +
@@ -974,6 +1322,7 @@ main(argc, argv)
                                 sprintf(s, "\001T%lu", lr->ltime);
 				addSDES(RTCP_SDES_PRIV, s);
 
+#ifdef NEEDED
                                 /* If we're over the packet size limit,
                                    let the user know there's more to be
 				   retrieved starting at the given offset. */
@@ -982,6 +1331,7 @@ main(argc, argv)
                                     sprintf(s, "\001M%d", scandex);
 				    addSDES(RTCP_SDES_PRIV, s);
 				}
+#endif
 				*ap++ = RTCP_SDES_END;
 
 				/* Pad to next 32 bit boundary. */
@@ -989,14 +1339,15 @@ main(argc, argv)
 				while (((ap - b) & 3) != 0) {
 				    *ap++ = RTCP_SDES_END;
 				}
+				if ((ap - b) > MaxReplyPacket) {
+				    ap = pap;
+				    break;
+				}
 #ifdef RationalWorld
 				rp->common.count++;
 #else
 				(((unsigned char *) rp)[0])++;
 #endif
-				if ((ap - b) > MaxReplyPacket) {
-				    break;
-				}
 			    }
 			    lr = lr->next;
 			    scandex++;
