@@ -31,8 +31,11 @@ static struct destination *dests = NULL, *dtail;
 static int compressing = FALSE;       /* Compress sound: simple 2X */
 static int gsmcompress = TRUE;	      /* GSM compress buffers */
 static int lpccompress = FALSE;       /* LPC compress buffers */
+static int lpc10compress = FALSE;     /* LPC-10 compress buffers */
 static int adpcmcompress = FALSE;     /* ADPCM compress buffers */
 static int toasted = FALSE;	      /* Sending already-compressed file ? */
+static int robust = 1;		      /* Robust duplicate packets mode */
+static int rseq = 0;		      /* Robust mode packet sequence number */
 
 #define DEFAULT_SQUELCH 4096	      /* Default squelch level */
 static int squelch = 0; 	      /* Squelch level if > 0 */
@@ -189,6 +192,32 @@ static void lpccomp(sb)
     sb->buffer.buffer_len = l + sizeof(short);
 }
 
+/*  LPC10COMP  --  Compress the contents of a sound buffer using LPC-10.  */
+
+static void lpc10comp(sb)
+  struct soundbuf *sb;
+{
+    char *dp = (char *) sb->buffer.buffer_val;
+    unsigned char *src = ((unsigned char *) sb->buffer.buffer_val);
+
+    sb->compression |= fCompLPC10;
+    sb->buffer.buffer_len = lpc10encode(src, dp, sb->buffer.buffer_len);
+}
+
+/*  LPC10STUFF	--  Stuff last 16 bytes of LPC10 packet in sendinghost. */
+
+static int lpc10stuff(sb, pktlen)
+  struct soundbuf *sb;
+  int pktlen;
+{
+    if ((sb->compression & fCompLPC10) && (sb->buffer.buffer_len > 16)) {
+	bcopy(((char *) sb) + (pktlen - (sizeof sb->sendinghost)),
+	    sb->sendinghost, sizeof sb->sendinghost);
+	pktlen -= sizeof sb->sendinghost;
+    }
+    return pktlen;
+}
+
 /*  SENDRTPCTRL  --  Send RTP or VAT control message, encrypting if
 		     necessary.  */
 
@@ -380,7 +409,7 @@ static void timertick()
 #ifdef MULTICAST
 	if (IN_MULTICAST(d->name.sin_addr.s_addr)) {
 	    setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL,
-		&(d->time_to_live), sizeof d->time_to_live);
+		(char *) &(d->time_to_live), sizeof d->time_to_live);
 	    if (debugging) {
                 fprintf(stderr, "Multicasting ID/SDES with scope of %d to %s.\n",
 		    d->time_to_live, inet_ntoa(d->name.sin_addr));
@@ -434,6 +463,10 @@ static int sendpkt(sb)
 	lpccomp(sb);
     }
 
+    if (lpc10compress && !toasted) {
+	lpc10comp(sb);
+    }
+
     if (hasFace) {
 	sb->compression |= fFaceOffer;
     }
@@ -457,7 +490,7 @@ static int sendpkt(sb)
 #ifdef MULTICAST
 	if (IN_MULTICAST(d->name.sin_addr.s_addr)) {
 	    setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL,
-		&(d->time_to_live), sizeof d->time_to_live);
+		(char *) &(d->time_to_live), sizeof d->time_to_live);
 	    if (debugging) {
                 fprintf(stderr, "Multicasting with scope of %d to %s.\n",
 		    d->time_to_live, inet_ntoa(d->name.sin_addr));
@@ -592,7 +625,7 @@ static int sendpkt(sb)
 	    if (d->otp != NULL) {
 		if (debugging) {
                     fprintf(stderr, "Encrypting %d bytes with key file.\r\n",
-			    ebuf.buffer.buffer_len);
+			    slen);
 		}
 		for (i = 0; i < slen; i ++) {
 		    ebuf.buffer.buffer_val[i] ^= d->otp[i];
@@ -601,23 +634,59 @@ static int sendpkt(sb)
 	    }
 
 	    if (!rtp && !vat) {
+		if (lpc10compress && !toasted) {
+		    pktlen = lpc10stuff(&ebuf, pktlen);
+		    if (robust > 1) {
+			ebuf.compression |= fCompRobust;
+			roSeq(&ebuf) = rseq;
+			rseq = (rseq + 1) & 0xFF;
+		    }
+		}
 		ebuf.compression = htonl(ebuf.compression);
 		ebuf.buffer.buffer_len = htonl(ebuf.buffer.buffer_len);
 	    }
-	    if (sendto(sock, (char *) &ebuf, pktlen,
-		0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
-                perror("sending datagram message");
-		return FALSE;
+	    for (i = 0; i < (lpc10comp ? robust : 1); i++) {
+		if (sendto(sock, (char *) &ebuf, pktlen,
+		    0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
+                    perror("sending datagram message");
+		    return FALSE;
+		}
 	    }
 	} else {
+	    int i, wasf = FALSE;
+
 	    if (!vat && !rtp) {
+		wasf = TRUE;
+		if (lpc10compress && !toasted) {
+		    pktlen = lpc10stuff(sb, pktlen);
+		    if (robust > 1) {
+			sb->compression |= fCompRobust;
+			sb->buffer.buffer_len |= ((long) rseq) << 24;
+			rseq = (rseq + 1) & 0xFF;
+		    }
+		}
 		sb->compression = htonl(sb->compression);
 		sb->buffer.buffer_len = htonl(sb->buffer.buffer_len);
 	    }
-	    if (sendto(sock, (char *) sb, pktlen,
-		0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
-                perror("sending datagram message");
-		return FALSE;
+
+	    for (i = 0; i < (lpc10comp ? robust : 1); i++) {
+		if (sendto(sock, (char *) sb, pktlen,
+		    0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
+                    perror("sending datagram message");
+		    return FALSE;
+		}
+	    }
+	    if (wasf) {
+		sb->compression = ntohl(sb->compression);
+		sb->buffer.buffer_len = ntohl(sb->buffer.buffer_len);
+		if (sb->compression & fCompRobust) {
+		    sb->compression &= ~fCompRobust;
+		    sb->buffer.buffer_len &= 0xFFFFFFL;
+		}
+		if ((sb->compression & fCompLPC10) &&
+		    (sb->buffer.buffer_len > 16)) {
+		    pktlen += sizeof(sb->sendinghost);
+		}
 	    }
 	}
     }
@@ -840,6 +909,8 @@ static int sendfile(f)
 	    sound_packet -= 4;		  /* Leave room for state at the end */
 	} else if (lpccompress) {
 	    sound_packet = 10 * LPC_FRAME_SIZE;
+	} else if (lpc10compress) {
+	    sound_packet = compressing ? 3600 : 1800;
 	}
     }
 
@@ -919,12 +990,19 @@ static int sendfile(f)
 		netbuf.compression |= fCompGSM;
 	    } else {
 		if (compressing) {
+#ifdef OLD2X
 		    int i;
 
 		    nread /= 2;
 		    for (i = 1; i < nread; i++) {
 			buf[i] = buf[i * 2];
 		    }
+#else
+		    int is = nread, os = nread / 2;
+
+		    rate_flow(buf, buf, &is, &os);
+		    nread = os;
+#endif
 		    netbuf.compression |= fComp2X;
 		}
 		netbuf.buffer.buffer_len = nread;
@@ -1239,12 +1317,19 @@ perror("Read poll"); fflush(stderr);
 
 		    ring = FALSE;
 		    if (compressing) {
+#ifdef OLD2X
 			int i;
 
 			soundel /= 2;
 			for (i = 1; i < soundel; i++) {
 			    buf[i] = buf[i * 2];
 			}
+#else
+			int is = soundel, os = soundel / 2;
+
+			rate_flow(buf, buf, &is, &os);
+			soundel = os;
+#endif
 			netbuf.compression |= fComp2X;
 		    }
 		    netbuf.buffer.buffer_len = soundel;
@@ -1334,6 +1419,16 @@ static void makeSessionKey(key, seed, mode)
     close_idearand();
 }
 
+/*  PROG_NAME  --  Extract program name from argv[0].  */
+
+static char *prog_name(arg)
+  char *arg;
+{
+    char *cp = strrchr(arg, '/');
+
+    return (cp != NULL) ? cp + 1 : arg;
+}
+
 /*  USAGE  --  Print how-to-call information.  */
 
 static void usage()
@@ -1356,6 +1451,7 @@ static void usage()
     V fprintf(stderr, "           -Kkey      DES encrypt with key\n");
     V fprintf(stderr, "           -L         Remote loopback\n");
     V fprintf(stderr, "           -LPC       LPC compression\n");
+    V fprintf(stderr, "           -LPC10Rn   LPC-10 compression, n copies\n");
     V fprintf(stderr, "     *     -M         Manual record gain control\n");
     V fprintf(stderr, "           -N         Do not compress subsequent sound\n");
     V fprintf(stderr, "           -Ofile     Use file as key file\n");
@@ -1387,11 +1483,21 @@ main(argc, argv)
     char md5key[16], md5key1[16];
     char s[1024];
 
-    progname = argv[0];
+    progname = prog_name(argv[0]);
     gethostname(hostname, sizeof hostname);
     if (strlen(hostname) > 15) {
 	hostname[15] = 0;
     }
+#ifdef LINUX_FPU_FIX
+    __setfpucw(_FPU_IEEE);	      /* Mask floating point interrupts to
+					 enable standard IEEE processing.
+					 Current libc releases do this by
+					 default, so this is needed only on
+					 older libraries. */
+#endif
+#ifndef OLD2X
+    rate_start(8000, 4000);
+#endif
     gsmh = gsm_create();
 
     /* Initialise LPC decoding. */
@@ -1401,6 +1507,7 @@ main(argc, argv)
 	return 1;
     }
     lpc_init(&lpcc);
+    lpc10init();
 
 #ifdef sgi
     usticks = 1000000 / CLK_TCK;
@@ -1500,7 +1607,7 @@ main(argc, argv)
 
                 case 'F':             /* -F -- ADPCM compression */
 		    adpcmcompress = TRUE;
-		    gsmcompress = lpccompress = FALSE;
+		    gsmcompress = lpccompress = lpc10compress = FALSE;
 		    break;
 
                 case 'G':             /* -G  --  Automatic gain control */
@@ -1572,11 +1679,29 @@ main(argc, argv)
 		    }
 		    break;
 
-                case 'L':             /* -L    --  Remote loopback */
-				      /* -LPC  --  LPC compress sound */
+                case 'L':             /* -L      --  Remote loopback */
+				      /* -LPC	 --  LPC compress sound */
+				      /* -LPC10  --  LPC10 compress sound */
                     if (ucase(op[1]) == 'P') {
-			lpccompress = TRUE;
-			gsmcompress = adpcmcompress = FALSE;
+                        if (strlen(op) > 3 && op[3] == '1') {
+			    lpc10compress = TRUE;
+			    gsmcompress = adpcmcompress = lpccompress = FALSE;
+			    robust = 1;
+                            if (strlen(op) > 5 && ucase(op[5]) == 'R') {
+				robust = 2;
+				if (strlen(op) > 6 && isdigit(op[6])) {
+				    robust = atoi(op + 6);
+				    if (robust <= 0) {
+					robust = 1;
+				    } else if (robust > 4) {
+					robust = 4;
+				    }
+				}
+			    }
+			} else {
+			    lpccompress = TRUE;
+			    gsmcompress = adpcmcompress = lpc10compress = FALSE;
+			}
 		    } else {
 			loopback = TRUE;
 		    }
@@ -1590,6 +1715,7 @@ main(argc, argv)
 		    compressing = FALSE;
 		    gsmcompress = FALSE;
 		    lpccompress = FALSE;
+		    lpc10compress = FALSE;
 		    adpcmcompress = FALSE;
 		    break;
 
@@ -1669,7 +1795,7 @@ main(argc, argv)
 
                 case 'T':             /* -T -- Telephone (GSM) compression */
 		    gsmcompress = TRUE;
-		    lpccompress = adpcmcompress = FALSE;
+		    lpccompress = lpc10compress = adpcmcompress = FALSE;
 		    break;
 
                 case 'U':             /* -U  --  Print usage information */
@@ -1799,6 +1925,10 @@ main(argc, argv)
                     fprintf(stderr, "Simple (-C) compression cannot be used with\n   -RTP or -VAT protocols.\n");
 		    return 2;
 		}
+		if (lpc10compress) {
+                    fprintf(stderr, "LPC-10 (-LPC10) compression cannot be used with\n   -RTP or -VAT protocols.\n");
+		    return 2;
+		}
 		if (ring) {
                     fprintf(stderr, "Warning: ring not implemented in -RTP and -VAT protocols.\n");
 		}
