@@ -46,14 +46,18 @@ static struct sockaddr_in lookhost;   /* Look who's listening host, if any */
 static int lookwho = FALSE;           /* Contact look who's listening ? */
 static char *sdes = NULL;	      /* RTP SDES packet */
 static int sdesl;		      /* RTP SDES packet length */
-static int lwlport = Internet_Port + 2; /* Look Who's Listening port */
 static unsigned long ssrc;	      /* RTP synchronisation source identifier */
 static int lwltimer;		      /* Seconds before next LWL retransmit */
+static int lwlonly = FALSE;	      /* LWL broadcast only ? */
 static int actives = 0; 	      /* Currently active hosts */
 static int faceTransferActive = 0;    /* Face transfers in progress */
 
 #define LWL_RETRANSMIT	(10 * 60 * 1000000L) /* Microseconds between LWL updates */
 
+static struct in_addr lwl_sites[LWL_MAX_SITES]; /* LWL site addresses */
+static long lwl_ports[LWL_MAX_SITES]; /* Ports for LWL hosts */
+static int lwl_nsites = 0;	      /* Number of LWL sites published on */
+
 static FILE *facefile = NULL;         /* User's face image */
 static int facePlace = 0;	      /* Placement for next face viewer */
 static int facesDisplayed = 0;	      /* Face viewers currently active */
@@ -65,7 +69,11 @@ struct connection *conn = NULL;       /* Chain of current connections */
 #define TickTock    (10 * 1000000L)   /* Alarm interval in microseconds */
 #define TockTock    (60 * 1000000L)   /* Alarm interval when no connections open */
 
+#ifdef AUDIO_RELEASE
+#define AudioRelease (AUDIO_RELEASE * 1000000L) /* Release audio when idle this long */
+#else
 #define AudioRelease (2 * TickTock)   /* Release audio when idle this long */
+#endif
 
 static int timerStep = TockTock;      /* Current timer step */
 
@@ -105,7 +113,7 @@ static void lpcdecomp(sb, c)
 {
     int i, j, l = 0;
     char *dpx = ((char *) sb->buffer.buffer_val) + sizeof(short);
-    static char dcb[BUFL];
+    char dcb[BUFL];
     short declen = ntohl(*((short *) sb->buffer.buffer_val));
 
     if (declen <= 0 || declen > (10 * LPC_FRAME_SIZE)) {
@@ -120,6 +128,21 @@ static void lpcdecomp(sb, c)
     sb->buffer.buffer_len = declen;
 }
 
+/*  LPC10DECOMP  --  Uncompress the contents of a sound buffer using LPC10.  */
+
+static void lpc10decomp(sb, c)
+  struct soundbuf *sb;
+  struct connection *c;
+{
+    int j;
+    char *dpx = ((char *) sb->buffer.buffer_val);
+    char dcb[BUFL];
+
+    j = lpc10decode(dpx, dcb, sb->buffer.buffer_len);
+    bcopy(dcb, sb->buffer.buffer_val, j);
+    sb->buffer.buffer_len = j;
+}
+
 /*  ETIME  --  Edit time and date for log messages.  */
 
 static char *etime()
@@ -152,15 +175,19 @@ static char *compressionType(msg)
 	   ((msg->compression & (fComp2X | fCompLPC)) ==
 				(fComp2X | fCompLPC)) ?
                                 "LPC+2X compressed" :
+	   ((msg->compression & (fComp2X | fCompLPC10)) ==
+				(fComp2X | fCompLPC10)) ?
+                                "LPC10+2X compressed" :
 	   ((msg->compression & (fComp2X | fCompVOX)) ==
 				(fComp2X | fCompVOX)) ?
                                 "VOX+2X compressed" :
            ((msg->compression & fCompADPCM) ? "ADPCM compressed" :
            ((msg->compression & fCompLPC) ? "LPC compressed" :
+           ((msg->compression & fCompLPC10) ? "LPC10 compressed" :
            ((msg->compression & fComp2X) ? "2X compressed" :
            ((msg->compression & fCompGSM) ? "GSM compressed" :
            ((msg->compression & fCompVOX) ? "VOX compressed" :
-                                            "uncompressed")))));
+                                            "uncompressed"))))));
 }
 
 /*  MAKESESSIONKEY  --	Generate session key.  */
@@ -233,46 +260,54 @@ static void windtimer()
 }
 
 /*  SENDLWLMESSAGE  --	If enabled, send a message identifying us
-                        to the selected Look Who's Listening server.  */
+                        to each selected Look Who's Listening server.  */
 
 static void sendLwlMessage(dobye)
   int dobye;
 {
-    int sock;
+    int i, sock;
     struct sockaddr_in name;
 
-    sock = socket(AF_INET, SOCK_STREAM, 0);
-    if (sock < 0) {
-        perror("opening look who's listening socket");
-	sdes = NULL;
-	return;
-    }
-
-    if (connect(sock, (struct sockaddr *) &(lookhost), sizeof lookhost) >= 0) {
-	if (dobye) {
-	    char v[1024];
-	    int l;
-
-            l = rtp_make_bye(v, ssrc, "Exiting sfspeaker", FALSE);
-	    if (send(sock, v, l, 0) < 0) {
-                perror("sending look who's listening BYE packet");
+    for (i = 0; i < lwl_nsites; i++) {
+	if (lwl_ports[i] >= 0) {
+	    sock = socket(AF_INET, SOCK_STREAM, 0);
+	    if (sock < 0) {
+                perror("opening look who's listening socket");
+		sdes = NULL;
+		return;
 	    }
-	} else {
-	    if (send(sock, (char *) sdes, sdesl, 0) < 0) {
-                perror("sending look who's listening source ID message");
+
+	    lookhost.sin_port = htons(lwl_ports[i]);
+	    bcopy((char *) (&lwl_sites[i]), (char *) &lookhost.sin_addr.s_addr,
+		  sizeof lookhost.sin_addr.s_addr);
+
+	    if (connect(sock, (struct sockaddr *) &(lookhost), sizeof lookhost) >= 0) {
+		if (dobye) {
+		    char v[1024];
+		    int l;
+
+                    l = rtp_make_bye(v, ssrc, "Exiting sfspeaker", FALSE);
+		    if (send(sock, v, l, 0) < 0) {
+                        perror("sending look who's listening BYE packet");
+		    }
+		} else {
+		    if (send(sock, (char *) sdes, sdesl, 0) < 0) {
+                        perror("sending look who's listening source ID message");
+		    }
+		}
+	    } else {
+                perror("connecting look who's listening socket");
+
+		/* If we get a connection refused reply, this server is
+                   no longer running Look Who's Listening.  To avoid
+		   bombarding it with subsequent futile requests,
+		   null out the port. */
+
+		if (errno == ECONNREFUSED) {
+		    lwl_ports[i] = -1;
+		}
 	    }
 	}
-    } else {
-        perror("connecting look who's listening socket");
-
-	/* If we get a connection refused reply, this server is
-           no longer running Look Who's Listening.  To avoid
-	   bombarding it with subsequent futile requests,
-	   null out the ID message. */
-
-	if (errno == ECONNREFUSED) {
-	    sdes = NULL;
-	}
     }
     lwltimer = LWL_RETRANSMIT;
     close(sock);
@@ -305,6 +340,7 @@ static void release()
 	    }
 	    c->con_protocol = PROTOCOL_UNKNOWN;
 	    c->con_uname[0] = c->con_email[0] = 0;
+	    c->con_rseq = -1;
 
 	    /* If a face file transfer is in progress, shut it down and
 	       discard the incomplete face file. */
@@ -573,11 +609,12 @@ static void playbuffer(msg, c)
 
     if ((msg->compression & fEncOTP) && (curotp != NULL)) {
 	int i;
+	LONG slen = (len + 7) & (~7);
 
 	if (Debug) {
             fprintf(stderr, "%s: decrypting %d bytes with key file.\n", prog, len);
 	}
-	for (i = 0; i < len; i ++) {
+	for (i = 0; i < slen; i ++) {
 	    val[i] ^= curotp[i];
 	}
     }
@@ -612,13 +649,14 @@ static void playbuffer(msg, c)
 
     if ((msg->compression & fEncDES) && deskey[0]) {
 	int i;
+	LONG slen = (len + 7) & (~7);
 
 	setkey(deskey + 1);
 
 	if (Debug) {
-            fprintf(stderr, "%s: decrypting %d bytes with DES key.\n", prog, len);
+            fprintf(stderr, "%s: decrypting %d bytes with DES key.\n", prog, slen);
 	}
-	for (i = 0; i < len; i += 8) {
+	for (i = 0; i < slen; i += 8) {
 	    bcopy(val + i, tbuf, 8);
 	    dedes(val + i);
 
@@ -657,7 +695,13 @@ static void playbuffer(msg, c)
 	len = msg->buffer.buffer_len;
     }
 
+    if (msg->compression & fCompLPC10) {
+	lpc10decomp(msg, c);
+	len = msg->buffer.buffer_len;
+    }
+
     if (msg->compression & fComp2X) {
+#ifdef OLD2X
 	int i;
 	register char *ab = auxbuf;
 
@@ -673,6 +717,12 @@ static void playbuffer(msg, c)
 	    *ab++ = *val++;
 	}
 	len *= 2;
+#else
+	int is = len, os = len * 2;
+
+	rate_flow(val, auxbuf, &is, &os);
+	len = os;
+#endif
 	val = auxbuf;
     }
 
@@ -682,6 +732,16 @@ static void playbuffer(msg, c)
     }
 }
 
+/*  PROGNAME  --  Extract program name from argv[0].  */
+
+static char *progname(arg)
+  char *arg;
+{
+    char *cp = strrchr(arg, '/');
+
+    return (cp != NULL) ? cp + 1 : arg;
+}
+
 /*  USAGE  --  Print how-to-call information.  */
 
 static void usage()
@@ -705,6 +765,7 @@ static void usage()
     V fprintf(stderr, "           -R[+]file        Record [append] sound in file\n");
     V fprintf(stderr, "           -U               Print this message\n");
     V fprintf(stderr, "           -Vtimeout        Show hostnames that connect\n");
+    V fprintf(stderr, "           -W               Publish LWL information for sfvod\n");
 #ifdef HEXDUMP
     V fprintf(stderr, "           -X               Dump packets in hex\n");
 #endif
@@ -733,7 +794,7 @@ main(argc, argv)
     };
     static struct auhdr afh = { ".snd", sizeof(struct auhdr), 1, 1, 8000, 1 };
 
-    prog = argv[0];
+    prog = progname(argv[0]);
 
     /* First pass option processing.  We have to first scan
        the options to handle any which affect creation of the
@@ -753,14 +814,27 @@ main(argc, argv)
 
 	    switch (opt) {
 
+                case 'D':             /* -D  --  Force debug output */
+		    debugforce = TRUE;
+		    break;
+
                 case 'P':             /* -Pport  --  Port to listen on */
-		    whichport = atoi(op + 1);
+		    if (op[1] != 0) {
+			whichport = atoi(op + 1);
+		    }
 		    break;
 
                 case 'U':             /* -U  --  Print usage information */
                 case '?':             /* -?  --  Print usage information */
 		    usage();
 		    return 0;
+
+                case 'W':           /* -Wport  --  Publish LWL information for sfvod */
+		    lwlonly = TRUE;
+		    if (op[1] != 0) {
+			whichport = atoi(op + 1);
+		    }
+		    break;
 	    }
 	} else {
 	    usage();
@@ -768,292 +842,321 @@ main(argc, argv)
 	}
     }
 
-    /* Create the sockets from which to read */
+    if (!lwlonly) {
 
-    sock = socket(AF_INET, SOCK_DGRAM, 0);
-    if (sock < 0) {
-        perror("opening data socket");
-	return 1;
-    }
+	/* Create the sockets from which to read */
 
-    ssock = socket(AF_INET, SOCK_DGRAM, 0);
-    if (ssock < 0) {
-        perror("opening control socket");
-	return 1;
-    }
+	sock = socket(AF_INET, SOCK_DGRAM, 0);
+	if (sock < 0) {
+            perror("opening data socket");
+	    return 1;
+	}
 
-    /* Create name with wildcards. */
+	ssock = socket(AF_INET, SOCK_DGRAM, 0);
+	if (ssock < 0) {
+            perror("opening control socket");
+	    return 1;
+	}
 
-    name.sin_family = AF_INET;
-    name.sin_addr.s_addr = INADDR_ANY;
-    name.sin_port = htons(whichport);
-    if (bind(sock, (struct sockaddr *) &name, sizeof name) < 0) {
-        perror("binding data socket");
-	return 1;
-    }
-    name.sin_port = htons(whichport + 1);
-    if (bind(ssock, (struct sockaddr *) &name, sizeof name) < 0) {
-        perror("binding control socket");
-	return 1;
-    }
-    name.sin_port = htons(whichport);
+	/* Create name with wildcards. */
 
-    /*	Process command line options.  */
+	name.sin_family = AF_INET;
+	name.sin_addr.s_addr = INADDR_ANY;
+	name.sin_port = htons(whichport);
+	if (bind(sock, (struct sockaddr *) &name, sizeof name) < 0) {
+            perror("binding data socket");
+	    return 1;
+	}
+	name.sin_port = htons(whichport + 1);
+	if (bind(ssock, (struct sockaddr *) &name, sizeof name) < 0) {
+            perror("binding control socket");
+	    return 1;
+	}
+	name.sin_port = htons(whichport);
 
-    pgppass = getenv("PGPPASS");
+	/*  Process command line options.  */
+
+        pgppass = getenv("PGPPASS");
 
 #if Internet_Port != 2074
-    /* This is a reminder to the author who frequently tests with
-       non-standard ports to avoid inadvertently releasing a
-       production version configured for the wrong port. */
-    fprintf(stderr, "%s: warning, listening on non-standard Internet port %d\n",
-	prog, Internet_Port);
+	/* This is a reminder to the author who frequently tests with
+	   non-standard ports to avoid inadvertently releasing a
+	   production version configured for the wrong port. */
+        fprintf(stderr, "%s: warning, listening on non-standard Internet port %d\n",
+	    prog, Internet_Port);
 #endif
 
-    for (i = 1; i < argc; i++) {
-	char *op, opt;
+	for (i = 1; i < argc; i++) {
+	    char *op, opt;
 
-	op = argv[i];
-        if (*op == '-') {
-	    opt = *(++op);
-	    if (islower(opt)) {
-		opt = toupper(opt);
-	    }
+	    op = argv[i];
+            if (*op == '-') {
+		opt = *(++op);
+		if (islower(opt)) {
+		    opt = toupper(opt);
+		}
 
-	    switch (opt) {
+		switch (opt) {
 
-                case 'A':             /* -A"file command"  --  Create reply command on connect */
-		    {
-                        char *cp = strchr(op + 1, ' ');
+                    case 'A':             /* -A"file command"  --  Create reply command on connect */
+			{
+                            char *cp = strchr(op + 1, ' ');
 
-			if (cp != NULL) {
-			    replycmd = cp + 1;
-			    *cp = 0;
+			    if (cp != NULL) {
+				replycmd = cp + 1;
+				*cp = 0;
+			    }
+			    replyfile = op + 1;
 			}
-			replyfile = op + 1;
-		    }
-		    break;
+			break;
 
-                case 'D':             /* -D  --  Force debug output */
-		    debugforce = TRUE;
-		    break;
+                    case 'I':             /* -Ikey  --  Set IDEA key */
+			if (strlen(op + 1) == 0) {
+			    ideakey[0] = FALSE;
+			} else {
+			    struct MD5Context md5c;
 
-                case 'I':             /* -Ikey  --  Set IDEA key */
-		    if (strlen(op + 1) == 0) {
-			ideakey[0] = FALSE;
-		    } else {
-			struct MD5Context md5c;
-
-			MD5Init(&md5c);
-			MD5Update(&md5c, op + 1, strlen(op + 1));
-			MD5Final(ideakey + 1, &md5c);
-			ideakey[0] = TRUE;
-		    }
-		    break;
-
-                case 'K':             /* -Kkey  --  Set DES key */
-		    desinit(1);       /* Initialise the DES library */
-		    if (strlen(op + 1) == 0) {
-			deskey[0] = rtpdeskey[0] = vatdeskey[0] = FALSE;
-		    } else {
-			struct MD5Context md5c;
-			char md5key[16], algorithm[16];
-
-			MD5Init(&md5c);
-			MD5Update(&md5c, op + 1, strlen(op + 1));
-			MD5Final(md5key, &md5c);
-			for (j = 0; j < 8; j++) {
-			    deskey[j + 1] = (char)
-					  ((md5key[j] ^ md5key[j + 8]) & 0x7F);
+			    MD5Init(&md5c);
+			    MD5Update(&md5c, op + 1, strlen(op + 1));
+			    MD5Final(ideakey + 1, &md5c);
+			    ideakey[0] = TRUE;
 			}
-			deskey[0] = TRUE;
-			des_string_to_key(op + 1, (des_cblock *) (vatdeskey + 1));
-			string_DES_key(op + 1, rtpdeskey + 1, algorithm);
-                        if (strcmp(algorithm, "DES-CBC") != 0) {
-                            fprintf(stderr, "Unsupported encryption algorithm: %s.  Only DES-CBC is available.\n",
-				    algorithm);
-			    return 2;
-			}
-			rtpdeskey[0] = vatdeskey[0] = TRUE;
-		    }
-		    break;
+			break;
 
-#ifdef MULTICAST
-                case 'M':             /* -Mhost/ip -- Join multicast to name/IP number */
-		    {
-			struct ip_mreq m;
-			struct hostent *h;
-			long naddr;
+                    case 'K':             /* -Kkey  --  Set DES key */
+			desinit(1);	  /* Initialise the DES library */
+			if (strlen(op + 1) == 0) {
+			    deskey[0] = rtpdeskey[0] = vatdeskey[0] = FALSE;
+			} else {
+			    struct MD5Context md5c;
+			    char md5key[16], algorithm[16];
 
-                        /* If it's a valid IP number, use it.  Otherwise try to look
-			   up as a host name. */
-
-			if ((naddr = inet_addr(op + 1)) == -1) {
-			    h = gethostbyname(op + 1);
-			    if (h == 0) {
-                                fprintf(stderr, "%s: unknown multicast group\n", op + 1);
+			    MD5Init(&md5c);
+			    MD5Update(&md5c, op + 1, strlen(op + 1));
+			    MD5Final(md5key, &md5c);
+			    for (j = 0; j < 8; j++) {
+				deskey[j + 1] = (char)
+					      ((md5key[j] ^ md5key[j + 8]) & 0x7F);
+			    }
+			    deskey[0] = TRUE;
+			    des_string_to_key(op + 1, (des_cblock *) (vatdeskey + 1));
+			    string_DES_key(op + 1, rtpdeskey + 1, algorithm);
+                            if (strcmp(algorithm, "DES-CBC") != 0) {
+                                fprintf(stderr, "Unsupported encryption algorithm: %s.  Only DES-CBC is available.\n",
+					algorithm);
 				return 2;
 			    }
-			    bcopy((char *) h->h_addr, (char *) &naddr, sizeof naddr);
+			    rtpdeskey[0] = vatdeskey[0] = TRUE;
+			    if (Debug) {
+                                fprintf(stderr, "DES key:");
+				for (j = 0; j < 8; j++) {
+                                    fprintf(stderr, " %02X", (deskey[j + 1] & 0xFF));
+				}
+                                fprintf(stderr, "\n");
+                                fprintf(stderr, "RTP key:");
+				for (j = 0; j < 8; j++) {
+                                    fprintf(stderr, " %02X", (rtpdeskey[j + 1] & 0xFF));
+				}
+                                fprintf(stderr, "\n");
+                                fprintf(stderr, "VAT key:");
+				for (j = 0; j < 8; j++) {
+                                    fprintf(stderr, " %02X", (vatdeskey[j + 1] & 0xFF));
+				}
+                                fprintf(stderr, "\n");
+			    }
 			}
-			m.imr_multiaddr.s_addr = naddr;
-			m.imr_interface.s_addr = htons(INADDR_ANY);
-			if (setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
-			    (char *) &m, sizeof m) < 0) {
-                            perror("joining multicast group");
-			    return 2;
+			break;
+
+#ifdef MULTICAST
+                    case 'M':             /* -Mhost/ip -- Join multicast to name/IP number */
+			{
+			    struct ip_mreq m;
+			    struct hostent *h;
+			    long naddr;
+
+                            /* If it's a valid IP number, use it.  Otherwise try to look
+			       up as a host name. */
+
+			    if ((naddr = inet_addr(op + 1)) == -1) {
+				h = gethostbyname(op + 1);
+				if (h == 0) {
+                                    fprintf(stderr, "%s: unknown multicast group\n", op + 1);
+				    return 2;
+				}
+				bcopy((char *) h->h_addr, (char *) &naddr, sizeof naddr);
+			    }
+			    m.imr_multiaddr.s_addr = naddr;
+			    m.imr_interface.s_addr = htons(INADDR_ANY);
+			    if (setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+				(char *) &m, sizeof m) < 0) {
+                                perror("joining multicast group");
+				return 2;
+			    }
 			}
-		    }
-		    break;
+			break;
 #endif
 
-                case 'N':             /* -N  --  Disable remote ring requests */
-		    noring = TRUE;
-		    break;
+                    case 'N':             /* -N  --  Disable remote ring requests */
+			noring = TRUE;
+			break;
 
-                case 'O':             /* -Ofile -- Use file as key file */
-                    fp = fopen(op + 1, "r");
-		    if (fp == NULL) {
-                        perror("Cannot open key file");
-			return 2;
-		    }
-		    curotp = malloc(BUFL);
-		    if (curotp == NULL) {
-                        fprintf(stderr, "%s: Cannot allocate key file buffer.\n", prog);
-			return 2;
-		    }
-		    l = fread(curotp, 1, BUFL, fp);
-		    if (l == 0) {
-                        /* Idiot supplied void key file.  Give 'im
-			   what he asked for: no encryption. */
-			curotp[0] = 0;
-			l = 1;
-		    }
-		    fclose(fp);
-		    /* If the file is shorter than the maximum buffer
-		       we may need to encrypt, replicate the key until
-		       the buffer is filled. */
-		    j = l;
-		    k = 0;
-		    while (j < BUFL) {
-			curotp[j++] = curotp[k++];
-			if (k >= l) {
-			    k = 0;
-			}
-		    }
-		    break;
-
-                case 'Q':             /* -Q  --  Prevent debug output */
-		    debugforce = -1;
-		    break;
-
-                case 'R':             /* -Rfile  --  Answering machine record */
-		    {
-			char *fn = op + 1;
-			int append = FALSE;
-
-                        if (*fn == '+') {
-			    fn++;
-			    append = TRUE;
-			}
-                        record = fopen(fn, append ? "a" : "w");
-			if (record == NULL) {
-                            perror("Cannot create answering machine file");
+                    case 'O':             /* -Ofile -- Use file as key file */
+                        fp = fopen(op + 1, "r");
+			if (fp == NULL) {
+                            perror("Cannot open key file");
 			    return 2;
 			}
-			fseek(record, 0L, 2);
-			if (ftell(record) == 0) {
-			    fwrite(&afh, sizeof afh, 1, record);
+			curotp = malloc(BUFL);
+			if (curotp == NULL) {
+                            fprintf(stderr, "%s: Cannot allocate key file buffer.\n", prog);
+			    return 2;
 			}
-			fflush(record);
-		    }
-		    break;
-
-                case 'V':             /* -Vtimeout  --  Show hostnames that connect */
-		    showhosts = TRUE;
-		    if (op[1] != 0) {
-			int t = atoi(op + 1) * 1000000L;
-
-			if (t > 0) {
-			    if (t < (TickTock + 1)) {
-				t = TickTock + 1;
+			l = fread(curotp, 1, BUFL, fp);
+			if (l == 0) {
+                            /* Idiot supplied void key file.  Give 'im
+			       what he asked for: no encryption. */
+			    curotp[0] = 0;
+			    l = 1;
+			}
+			fclose(fp);
+			/* If the file is shorter than the maximum buffer
+			   we may need to encrypt, replicate the key until
+			   the buffer is filled. */
+			j = l;
+			k = 0;
+			while (j < BUFL) {
+			    curotp[j++] = curotp[k++];
+			    if (k >= l) {
+				k = 0;
 			    }
-			    hosttimeout = (t / TickTock) * TickTock;
 			}
-		    }
-		    break;
+			break;
+
+                    case 'Q':             /* -Q  --  Prevent debug output */
+			debugforce = -1;
+			break;
+
+                    case 'R':             /* -Rfile  --  Answering machine record */
+			{
+			    char *fn = op + 1;
+			    int append = FALSE;
+
+                            if (*fn == '+') {
+				fn++;
+				append = TRUE;
+			    }
+                            record = fopen(fn, append ? "a" : "w");
+			    if (record == NULL) {
+                                perror("Cannot create answering machine file");
+				return 2;
+			    }
+			    fseek(record, 0L, 2);
+			    if (ftell(record) == 0) {
+				fwrite(&afh, sizeof afh, 1, record);
+			    }
+			    fflush(record);
+			}
+			break;
+
+                    case 'V':             /* -Vtimeout  --  Show hostnames that connect */
+			showhosts = TRUE;
+			if (op[1] != 0) {
+			    int t = atoi(op + 1) * 1000000L;
+
+			    if (t > 0) {
+				if (t < (TickTock + 1)) {
+				    t = TickTock + 1;
+				}
+				hosttimeout = (t / TickTock) * TickTock;
+			    }
+			}
+			break;
 
 #ifdef HEXDUMP
-                case 'X':             /* -X  --  Dump packets in hex */
-		    hexdump = debugforce = TRUE;
-		    break;
+                    case 'X':             /* -X  --  Dump packets in hex */
+			hexdump = debugforce = TRUE;
+			break;
 #endif
 
-                case 'Z':             /* -Z"phrase"  --  Set PGP pass phrase */
-		    if (op[1] != 0) {
-			pgppass = strdup(op + 1);
-			op = argv[i];
-                        /* Zap the pass phrase in memory so "ps" etc. don't see it. */
-			while (*op != 0) {
-                            *op++ = ' ';
-			}
-		    } else {
-			char s[256];
+                    case 'Z':             /* -Z"phrase"  --  Set PGP pass phrase */
+			if (op[1] != 0) {
+			    pgppass = strdup(op + 1);
+			    op = argv[i];
+                            /* Zap the pass phrase in memory so "ps" etc. don't see it. */
+			    while (*op != 0) {
+                                *op++ = ' ';
+			    }
+			} else {
+			    char s[256];
 
-                        fprintf(stderr, "PGP secret key pass phrase: ");
-			fflush(stderr);
-			initscr();
-			noecho();
-			getstr(s);
-			pgppass = strdup(s);
-			echo();
-			endwin();
-                        fprintf(stderr, "\n");
-		    }
-		    break;
+                            fprintf(stderr, "PGP secret key pass phrase: ");
+			    fflush(stderr);
+			    initscr();
+			    noecho();
+			    getstr(s);
+			    pgppass = strdup(s);
+			    echo();
+			    endwin();
+                            fprintf(stderr, "\n");
+			}
+			break;
+		}
 	    }
 	}
-    }
 
 #ifdef HALF_DUPLEX
-    {
-	struct hostent *h;
-	char host[512];
+	{
+	    struct hostent *h;
+	    char host[512];
 
-	gethostname(host, sizeof host);
-	h = gethostbyname(host);
-	bcopy((char *) (h->h_addr), (char *) &localhost,
-	    sizeof localhost);
+	    gethostname(host, sizeof host);
+	    h = gethostbyname(host);
+	    bcopy((char *) (h->h_addr), (char *) &localhost,
+		sizeof localhost);
 #ifdef HDX_DEBUG
-	if (Debug) {
-            fprintf(stderr, "%s: local host %s = %s\n", prog, host,
-		   inet_ntoa(localhost));
+	    if (Debug) {
+                fprintf(stderr, "%s: local host %s = %s\n", prog, host,
+		       inet_ntoa(localhost));
+	    }
+#endif
 	}
 #endif
-    }
+
+#ifdef LINUX_FPU_FIX
+    __setfpucw(_FPU_IEEE);	      /* Mask floating point interrupts to
+					 enable standard IEEE processing.
+					 Current libc releases do this by
+					 default, so this is needed only on
+					 older libraries. */
 #endif
 
-    /* Initialise GSM decoding. */
+#ifndef OLD2X
+	rate_start(4000, 8000);
+#endif
 
-    gsmh = gsm_create();
+	/* Initialise GSM decoding. */
 
-    /* Initialise LPC decoding. */
+	gsmh = gsm_create();
 
-    if (!lpc_start()) {
-        fprintf(stderr, "Cannot allocate LPC decoding memory.\n");
-	return 1;
-    }
+	/* Initialise LPC decoding. */
 
-    /* Find assigned port value and print it. */
+	if (!lpc_start()) {
+            fprintf(stderr, "Cannot allocate LPC decoding memory.\n");
+	    return 1;
+	}
+	lpc10init();
 
-    length = sizeof(name);
-    if (getsockname(sock, (struct sockaddr *) &name, &length) < 0) {
-        perror("getting socket name");
-	return 1;
-    }
+	/* Find assigned port value and print it. */
+
+	length = sizeof(name);
+	if (getsockname(sock, (struct sockaddr *) &name, &length) < 0) {
+            perror("getting socket name");
+	    return 1;
+	}
 #ifdef SHOW_SOCKET
-    fprintf(stderr, "%s: socket port #%d\n", prog, ntohs(name.sin_port));
+        fprintf(stderr, "%s: socket port #%d\n", prog, ntohs(name.sin_port));
 #endif
+    }
 
     /* Contact look who's listening host, if requested. */
 
@@ -1061,34 +1164,66 @@ main(argc, argv)
     if (cp != NULL) {
 	struct hostent *h;
 	char md5key[16];
-	char *ep;
+	char *ep, *np;
+	long iadr;
+	int n;
 
-        if ((ep = strchr(cp, ':')) != NULL) {
-	    *ep = 0;
-	    lwlport = atoi(ep + 1);
+	makeSessionKey(md5key);
+	bcopy(md5key, (char *) &ssrc, sizeof ssrc);
+	sdesl = rtp_make_sdes(&sdes, ssrc, whichport, FALSE);
+	lookhost.sin_family = AF_INET;
+
+	while (lwl_nsites < LWL_MAX_SITES) {
+	    n = lwl_nsites;
+	    while (*cp != 0 && isspace(*cp)) {
+		cp++;
+	    }
+	    if (*cp == 0) {
+		break;
+	    }
+            if ((np = strchr(cp, ',')) != NULL) {
+		*np++ = 0;
+	    }
+	    lwl_ports[lwl_nsites] = Internet_Port + 2;
+            if ((ep = strchr(cp, ':')) != NULL) {
+		*ep = 0;
+		lwl_ports[lwl_nsites] = atoi(ep + 1);
+	    }
+	    if (isdigit(*cp) && (iadr = inet_addr(cp)) != -1) {
+		bcopy((char *) &iadr, (char *) (&lwl_sites[lwl_nsites]),
+		      sizeof iadr);
+		lwl_nsites++;
+	    } else {
+		h = gethostbyname(cp);
+		if (h != NULL) {
+		    bcopy((char *) (h->h_addr), 
+			  (char *) (&lwl_sites[lwl_nsites]),
+			  sizeof(unsigned long));
+		    lwl_nsites++;
+		} else {
+                    fprintf(stderr, "%s: warning, SPEAKFREE_LWL_TELL host %s unknown.\n",
+			prog, cp);
+		}
+	    }
+	    if (Debug && lwl_nsites > n) {
+                fprintf(stderr, "%s: publishing on LWL server %s: %s.\n", prog,
+		    inet_ntoa(lwl_sites[n]), cp);
+	    }
+	    if (np == NULL) {
+		break;
+	    }
+	    cp = np;
 	}
-	h = gethostbyname(cp);
-	if (h != NULL) {
-	    bcopy((char *) (h->h_addr), (char *) &lookhost.sin_addr.s_addr,
-		sizeof lookhost.sin_addr.s_addr);
-	    lookhost.sin_family = AF_INET;
-	    lookhost.sin_port = htons(lwlport);
-
-	    makeSessionKey(md5key);
-	    bcopy(md5key, (char *) &ssrc, sizeof ssrc);
-
-	    sdesl = rtp_make_sdes(&sdes, ssrc, whichport, FALSE);
+	if (lwl_nsites > 0) {
 	    sendLwlMessage(FALSE);
-	} else {
-            fprintf(stderr, "%s: warning, SPEAKFREE_LWL_TELL host %s unknown.\n",
-		prog, cp);
 	}
     }
 
     /*	See if the user has specified a face file.  If so,
 	try to open it.  */
 
-    {   char *cp = getenv("SPEAKFREE_FACE");
+    if (!lwlonly) {
+        char *cp = getenv("SPEAKFREE_FACE");
 
 	if (cp != NULL) {
             if ((facefile = fopen(cp, "r")) == NULL) {
@@ -1105,6 +1240,19 @@ main(argc, argv)
     signal(SIGCHLD, viewerterm);      /* Set signal to handle face viewer termination */
     windtimer();		      /* Set periodic status update  */
 
+    /* If all we're doing is broadcasting a periodic LWL update
+       for sfvod, simply sit in a timer loop forever and let the
+       SIGALRM handler take care of business. */
+
+    while (lwlonly) {
+	if (lwl_nsites == 0) {
+	    /* -W option but no valid sites found.  Exit with error
+	       status. */
+	    return 2;
+	}
+	sleep(120);
+    }
+
     /* Read from the socket. */
 
     while (TRUE) {
@@ -1189,6 +1337,7 @@ main(argc, argv)
 	    c->face_retry = 0;
 	    c->con_compmodes = -1;
 	    c->con_protocol = PROTOCOL_UNKNOWN;
+	    c->con_rseq = -1;
 	    c->con_reply_current = FALSE;
             bcopy("\221\007\311\201", c->con_session, 4);
 	    lpc_init(&c->lpcc);
@@ -1533,6 +1682,16 @@ main(argc, argv)
 	    sb.compression = ntohl(sb.compression);
 	    sb.buffer.buffer_len = ntohl(sb.buffer.buffer_len);
 
+	    if (sb.compression & fCompRobust) {
+		int aseq = (sb.buffer.buffer_len >> 24) & 0xFF;
+
+		if (aseq == c->con_rseq) {
+		    continue;
+		}
+		c->con_rseq = aseq;
+		sb.buffer.buffer_len &= 0xFFFFFFL;
+	    }
+
 	    /* Now if this is a valid Speak Freely packet (as
 	       opposed to a VAT packet masquerading as one, or
                an encrypted VAT or RTP packet we don't have the
@@ -1548,6 +1707,9 @@ main(argc, argv)
 		xbl = ((sb.buffer.buffer_len + 7) & (~7)) +
 		      (sizeof(struct soundbuf) - BUFL);
 	    }
+	    if ((sb.compression & fCompLPC10) && (sb.buffer.buffer_len >= 16)) {
+		xbl -= 16;
+	    }
 	    if (xbl != rll) {
 		if (Debug) {
 		    fprintf(stderr,
@@ -1812,8 +1974,6 @@ main(argc, argv)
 	   one he originated. */
 
 	if (sb.compression & fLoopBack) {
-	    long l = sb.buffer.buffer_len;
-
 	    bcopy((char *) &(from.sin_addr), (char *) &(name.sin_addr),
 		sizeof(struct in_addr));
 	    sb.compression &= ~fLoopBack;    /* Prevent infinite loopback */
@@ -1821,7 +1981,7 @@ main(argc, argv)
 	    sb.compression = htonl(sb.compression);
 	    sb.buffer.buffer_len = htonl(sb.buffer.buffer_len);
 	    if (sendto(sock, (char *) &sb,
-		(int) ((sizeof(struct soundbuf) - BUFL) + l),
+		(int) ((sizeof(struct soundbuf) - BUFL) + rll),
 		0, (struct sockaddr *) &(name), sizeof name) < 0) {
                 perror("sending datagram message");
 	    }
@@ -1844,6 +2004,16 @@ main(argc, argv)
 	}
 #endif
 
+        /* If this packet has been "stuffed" for maximum efficiency,
+	   un-stuff it at this point. */
+
+	if ((sb.compression & fCompLPC10) && (sb.buffer.buffer_len >= 16)) {
+	    bcopy(sb.sendinghost, (char *) &sb + rll,
+		  sizeof sb.sendinghost);
+	    rll += sizeof sb.sendinghost;
+	}
+
+
 	if ((sb.compression & fKeyPGP)) {
 	    char cmd[256], f[40], kmd[16];
 	    FILE *kfile;
